@@ -1,41 +1,28 @@
 from json.decoder import JSONDecodeError
 from typing import List, Union
 from urllib.parse import urljoin, urlparse, urlsplit, urlunparse, urlunsplit
+from packaging import version
+from contextlib import contextmanager
 
 import requests
 from requests import Session
 from urllib3 import disable_warnings
 from urllib3.exceptions import InsecureRequestWarning
 
-from .const import (
-    DEFAULT_KW_ROUND_PERSICION,
-    SUPPORTED_OPERATION_MODES,
-    SUPPORTED_POWERWALL_VERSIONS,
-    DeviceType,
-    GridState,
-    GridStatus,
-    LineStatus,
-    MeterType,
-    OperationMode,
-    User,
-)
-from .error import AccessDeniedError, ApiError, PowerwallUnreachableError
+from .const import (DEFAULT_KW_ROUND_PERSICION, SUPPORTED_OPERATION_MODES,
+                    SUPPORTED_POWERWALL_VERSIONS, DeviceType, GridState,
+                    GridStatus, LineStatus, MeterType, OperationMode, User,
+                    Version)
+from .error import AccessDeniedError, APIError, PowerwallUnreachableError, APIChangedError, PowerwallError
 from .helpers import convert_to_kw
-from .responses import (
-    CustomerRegistrationResponse,
-    ListPowerwallsResponse,
-    LoginResponse,
-    MeterDetailsResponse,
-    MetersAggregateResponse,
-    MetersResponse,
-    PowerwallsStatusResponse,
-    PowerwallStatusResponse,
-    SiteInfoResponse,
-    SitemasterResponse,
-    SolarsResponse,
-)
+from .responses import (CustomerRegistrationResponse, ListPowerwallsResponse,
+                        LoginResponse, MeterDetailsResponse,
+                        MetersAggregateResponse, MetersResponse,
+                        PowerwallsStatusResponse, PowerwallStatusResponse,
+                        SiteInfoResponse, SitemasterResponse, SolarsResponse,
+                        UpdateStatusResponse)
 
-VERSION = "0.2.5"
+VERSION = "0.2.7"
 
 
 class Powerwall(object):
@@ -47,6 +34,7 @@ class Powerwall(object):
         verify_ssl: bool = False,
         disable_insecure_warning: bool = True,
         dont_validate_response: bool = True,
+        pin_version: str = None,
     ):
 
         if disable_insecure_warning:
@@ -73,9 +61,11 @@ class Powerwall(object):
 
         self._token = None
 
+        self.pin_version(pin_version)
+
     def _process_response(self, response: str) -> dict:
         if response.status_code == 404:
-            raise ApiError(f"The url {response.request.path_url} returned error 404")
+            raise APIError(f"The url {response.request.path_url} returned error 404")
 
         if response.status_code == 401 or response.status_code == 403:
             response_json = None
@@ -94,13 +84,13 @@ class Powerwall(object):
         try:
             response_json = response.json()
         except JSONDecodeError:
-            raise ApiError(f"Error while decoding json of response: {response.text}")
+            raise APIError(f"Error while decoding json of response: {response.text}")
 
         if response_json is None:
             return {}
 
         if "error" in response_json:
-            raise ApiError(response_json["error"])
+            raise APIError(response_json["error"])
 
         return response_json
 
@@ -108,7 +98,7 @@ class Powerwall(object):
         self, path: str, needs_authentication: bool = False, headers: dict = {}
     ) -> dict:
         if needs_authentication and not self.is_authenticated():
-            raise ApiError(f"Authentication required to access {path}")
+            raise APIError(f"Authentication required to access {path}")
 
         try:
             response = self._http_session.get(
@@ -116,7 +106,7 @@ class Powerwall(object):
                 timeout=self._timeout,
                 headers=headers,
             )
-        except requests.exceptions.ConnectionError as e:
+        except (requests.exceptions.ConnectionError, requests.exceptions.ReadTimeout) as e:
             raise PowerwallUnreachableError(e)
 
         return self._process_response(response)
@@ -129,7 +119,7 @@ class Powerwall(object):
         headers: dict = {},
     ) -> dict:
         if needs_authentication and not self.is_authenticated():
-            raise ApiError(f"Authentication required to access {path}")
+            raise APIError(f"Authentication required to access {path}")
 
         try:
             response = self._http_session.post(
@@ -138,7 +128,7 @@ class Powerwall(object):
                 timeout=self._timeout,
                 headers=headers,
             )
-        except requests.exceptions.ConnectionError as e:
+        except (requests.exceptions.ConnectionError, requests.exceptions.ReadTimeout) as e:
             raise PowerwallUnreachableError(e)
 
         return self._process_response(response)
@@ -179,7 +169,7 @@ class Powerwall(object):
 
     def logout(self):
         if not self.is_authenticated():
-            raise ApiError("Must be logged in to log out")
+            raise APIError("Must be logged in to log out")
         # The api unsets the auth cookie and the token is invalidated
         self._get("logout", True)
 
@@ -257,7 +247,10 @@ class Powerwall(object):
 
     def get_device_type(self) -> DeviceType:
         """Returns the device type of the powerwall"""
-        return DeviceType(self._get("device_type")["device_type"])
+        if self._pin_version is None or self._pin_version >= Version.v1_46_0.value:
+            return self.get_status().device_type
+        else:
+            return DeviceType(self._get("device_type")["device_type"])
 
     def get_customer_registration(self) -> CustomerRegistrationResponse:
         return CustomerRegistrationResponse(
@@ -329,6 +322,9 @@ class Powerwall(object):
     def get_git_hash(self) -> str:
         return self.get_status().git_hash
 
+    def get_update_status(self) -> UpdateStatusResponse:
+        return UpdateStatusResponse(self._get("system/update/status", needs_authentication=True))
+
     def is_sending_to(self, meter: MeterType, rounded=True) -> bool:
         """Wrapper method for is_sending_to"""
         return self.get_meters().get(meter).is_sending_to()
@@ -349,3 +345,40 @@ class Powerwall(object):
 
     def set_dont_validate_response(self, value):
         self._dont_validate_response = value
+
+    @contextmanager
+    def no_verify(self):
+        self.set_dont_validate_response(True)
+        try:
+            yield
+        finally:
+            self.set_dont_validate_response(False)
+
+
+    def detect_and_pin_version(self) -> str:
+        """Does an unferified request to get powerwall version and pins it"""
+        with self.no_verify():
+            status = self.get_status()
+            if status.has_key("version"):
+                version = status.get("version")
+            else:
+                raise APIError(
+                    "Could not detect version because the status response does not return the version"
+                )
+
+        self.pin_version(version)
+
+    def pin_version(self, vers: Union[str, version.Version, Version]):
+        if vers is None:
+            self._pin_version = None
+        else:
+            if isinstance(vers, str):
+                self._pin_version = version.parse(vers)
+            elif isinstance(vers, Version):
+                self._pin_version = vers.value
+            else:
+                self._pin_version = vers
+
+    def get_pinned_version(self) -> version.Version:
+        return self._pin_version
+
